"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const finder_1 = require("@jscpd/finder");
const safe_1 = require("colors/safe");
const html_reporter_1 = require("@jscpd/html-reporter");
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const reporters = {
    xml: finder_1.XmlReporter,
    json: finder_1.JsonReporter,
    csv: finder_1.CSVReporter,
    markdown: finder_1.MarkdownReporter,
    consoleFull: finder_1.ConsoleFullReporter,
    html: html_reporter_1.default,
    console: finder_1.ConsoleReporter,
    silent: finder_1.SilentReporter,
    threshold: finder_1.ThresholdReporter,
    xcode: finder_1.XcodeReporter,
};
function registerReporters(options, detector) {
    options.reporters.forEach((reporter) => {
        if (reporter in reporters) {
            detector.registerReporter(new reporters[reporter](options));
        }
        else {
            try {
                const reporterClass = require(`@jscpd/${reporter}-reporter`).default;
                detector.registerReporter(new reporterClass(options));
            }
            catch (e) {
                try {
                    const reporterClass = require(`jscpd-${reporter}-reporter`).default;
                    detector.registerReporter(new reporterClass(options));
                }
                catch (e) {
                    console.log(safe_1.yellow(`warning: ${reporter} not installed (install packages named @jscpd/${reporter}-reporter or jscpd-${reporter}-reporter)`));
                    console.log(safe_1.grey(e.message));
                }
            }
        }
    });
}
exports.registerReporters = registerReporters;
//# sourceMappingURL=reporters.js.map